import * as React from "react";

import { CourseContext, CourseProvider } from "./useCourse";
import { observer, useComputed } from "mobx-react-lite";
import { useContext, useEffect, useMemo } from "react";

import { AppStoreContext } from "src/components/AppStoreContext";
import { Breakpoint } from "src/style";
import { LayoutContext } from "../../layout/Layout";
import { Loading } from "src/components/loading/Loading";
import { NavigationComponentProps } from "../../navigation/Navigator";
import queryString from "query-string";
import { styles } from "./Course.styles";
import { useDocumentTitle } from "../../components/useDocumentTitle";
import { useFetchCourseByCode } from "openfing-core/lib/hooks/useFetchCourse";
import { useHistory } from "src/hooks/useHistory";

export const Course: React.FunctionComponent<NavigationComponentProps<{ courseCode: string }>> = observer(props => {
	const { courseCode } = props.match.params;

	useDocumentTitle("Curso - OpenFING");

	const { history } = useHistory();
	const queryParams = useMemo<{ t?: string; cci?: string; ccn?: string; ccl?: string }>(
		() => queryString.parse(history.location.search),
		[history.location.search]
	);
	const appStore = useContext(AppStoreContext);
	const [fetchCourseByCodeState] = useFetchCourseByCode({ courseCode, autoFetch: true, forceFetch: false });

	const course = fetchCourseByCodeState.success ? fetchCourseByCodeState.course : undefined;
	const courseName = course ? course.name : undefined;
	const setLayoutOptions = useContext(LayoutContext);
	const contextRef = React.useRef<CourseContext>();

	useEffect(() => {
		setLayoutOptions({
			header: {
				title: courseName ? courseName : "",
				right: course && <styles.CourseButton course={course} />,
			},
		});
	}, [setLayoutOptions, courseName, course]);

	const courseClassList =
		course && course.classLists && course.classLists.length > 0
			? queryParams.ccl
				? course.classLists.find(ccl => ccl.id.toString() === queryParams.ccl)
				: course.classLists[0]
			: undefined;
	const classes = courseClassList ? courseClassList.classes : undefined;
	const courseClass = useComputed(() => {
		if (!classes) return undefined;

		if (queryParams.cci) return classes.find(c => c.id.toString() === queryParams.cci);

		if (queryParams.ccn) return classes.find(c => !!c.number && c.number.toString() === queryParams.ccn);

		return undefined;
	}, [classes, queryParams]);
	useEffect(() => {
		const { current } = contextRef;

		if (!current) return;

		current.fetchCourseByCodeState = fetchCourseByCodeState;
		current.course = course;
		current.courseClassList = courseClassList;
		current.courseClass = courseClass;
	}, [fetchCourseByCodeState, course, courseClassList, courseClass]);

	useEffect(() => {
		if (!contextRef.current) return;
		contextRef.current.urlHash = queryParams.t ? "#t=" + queryParams.t : history.location.hash;
	}, [queryParams.t, history.location.hash]);

	const detail = (queryParams.ccn || queryParams.cci || appStore.breakpoint > Breakpoint.xs) && (
		<styles.CourseClassDetail key="detail" />
	);

	const master = <styles.CourseClassMaster key="master" />;

	const content = (
		<styles.Wrapper>
			{appStore.breakpoint < Breakpoint.sm ? (
				<>
					{master}
					{detail}
				</>
			) : (
				<>
					{detail}
					{master}
				</>
			)}
		</styles.Wrapper>
	);

	return (
		<CourseProvider contextRef={contextRef}>
			{fetchCourseByCodeState.isLoading ? <Loading /> : fetchCourseByCodeState.success ? content : null}
		</CourseProvider>
	);
});
